@@ -137,7 +137,7 @@ BUILTIN_MASKNAMES <- c("AGAPS", "AMB", "RM", "TRF")
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Accessor methods.
+### Accessors.
 ###
 
 setMethod("length", "BSgenome", function(x) length(names(x)))
@@ -145,18 +145,6 @@ setMethod("length", "BSgenome", function(x) length(names(x)))
 setGeneric("sourceUrl", function(x) standardGeneric("sourceUrl"))
 setMethod("sourceUrl", "BSgenome", function(x) x@source_url)
 
-setMethod("SNPlocs_pkgname", "BSgenome",
-    function(x) SNPlocs_pkgname(x@injectSNPs_handler)
-)
-
-setMethod("SNPcount", "BSgenome",
-    function(x) SNPcount(x@injectSNPs_handler)
-)
-
-setMethod("SNPlocs", "BSgenome",
-    function(x, seqname) SNPlocs(x@injectSNPs_handler, seqname)
-)
-
 setGeneric("mseqnames", function(x) standardGeneric("mseqnames"))
 setMethod("mseqnames", "BSgenome",
     function(x)
@@ -248,6 +236,23 @@ setReplaceMethod("seqnames", "BSgenome",
 )
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### SNP related accessors
+###
+
+setMethod("SNPlocs_pkgname", "BSgenome",
+    function(x) SNPlocs_pkgname(x@injectSNPs_handler)
+)
+
+setMethod("snpcount", "BSgenome",
+    function(x) snpcount(x@injectSNPs_handler)
+)
+
+setMethod("snplocs", "BSgenome",
+    function(x, seqname, ...) snplocs(x@injectSNPs_handler, seqname, ...)
+)
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### BSgenome constructor
 ###
@@ -258,11 +263,14 @@ setReplaceMethod("seqnames", "BSgenome",
                                  seqnames)
 {
     seqlengths <- seqlengths(single_sequences)
-    if (!identical(names(seqlengths), seqnames))
+    if (length(seqnames) == 0L) {
+        seqnames <- names(seqlengths)
+    } else if (!identical(names(seqlengths), seqnames)) {
         stop("sequence names found in file '",
              seqlengthsFilepath(single_sequences),
              "' are not identical to 'seqnames'. ",
              "May be the data on disk is corrupted?")
+    }
     if (identical(circ_seqs, NA)) {
         is_circ <- NA
     } else {
@@ -274,38 +282,52 @@ setReplaceMethod("seqnames", "BSgenome",
             genome=provider_version)
 }
 
-### NOTE: In BioC 2.14, the 'seqs_pkgname' and 'masks_pkgname' BSgenome slots
-### have been replaced with the 'pkgname' slot but the 2 corresponding
-### arguments have been kept for backward compatibility.
-BSgenome <- function(organism, species, provider, provider_version,
+### NOTES:
+### - In BioC 2.14, the 'seqs_pkgname' and 'masks_pkgname' BSgenome slots
+###   were replaced with the 'pkgname' slot but the 2 corresponding
+###   arguments were kept for backward compatibility with existing
+###   BSgenome packages.
+### - In BioC 3.1, the 'species' argument was replaced with the 'common_name'
+###   argument but the former was kept for backward compatibility with
+###   existing BSgenome packages.
+BSgenome <- function(organism, common_name, provider, provider_version,
                      release_date, release_name, source_url,
                      seqnames, circ_seqs=NA, mseqnames,
                      seqs_pkgname, seqs_dirpath,
-                     nmask_per_seq=0, masks_pkgname=NA, masks_dirpath=NA)
+                     nmask_per_seq=0, masks_pkgname=NA, masks_dirpath=NA,
+                     species=NA_character_)
 {
-    fa_filename <- "single_sequences.fa"
-    fa_filepath <- file.path(seqs_dirpath, fa_filename)
-    farz_filename <- paste0(fa_filename, ".rz")
-    farz_filepath <- file.path(seqs_dirpath, farz_filename)
-    if (file.exists(farz_filepath)) {
-        single_sequences <- FastaNamedSequences(farz_filepath)
-    } else if (file.exists(fa_filepath)) {
-        single_sequences <- FastaNamedSequences(fa_filepath)
+    twobit_filename <- "single_sequences.2bit"
+    twobit_filepath <- file.path(seqs_dirpath, twobit_filename)
+    if (file.exists(twobit_filepath)) {
+        single_sequences <- TwobitNamedSequences(twobit_filepath)
     } else {
-        single_sequences <- RdaNamedSequences(seqs_dirpath, seqnames)
+        fa_filename <- "single_sequences.fa"
+        fa_filepath <- file.path(seqs_dirpath, fa_filename)
+        farz_filename <- paste0(fa_filename, ".rz")
+        farz_filepath <- file.path(seqs_dirpath, farz_filename)
+        if (file.exists(farz_filepath)) {
+            single_sequences <- FastaNamedSequences(farz_filepath)
+        } else if (file.exists(fa_filepath)) {
+            single_sequences <- FastaNamedSequences(fa_filepath)
+        } else {
+            single_sequences <- RdaNamedSequences(seqs_dirpath, seqnames)
+        }
     }
     seqinfo <- .makeBSgenomeSeqinfo(single_sequences,
                                     circ_seqs, provider_version,
                                     seqnames)
-    genome_description <- GenomeDescription(organism, species,
+    seqnames <- seqnames(seqinfo)
+    if (missing(common_name))
+        common_name <- species
+    genome_description <- GenomeDescription(organism, common_name,
                                             provider, provider_version,
                                             release_date, release_name,
                                             seqinfo)
     if (is.null(mseqnames))
         mseqnames <- character(0)
     multiple_sequences <- RdaCollection(seqs_dirpath, mseqnames)
-    user_seqnames <- seqnames(seqinfo)
-    names(user_seqnames) <- user_seqnames
+    names(user_seqnames) <- user_seqnames <- seqnames
     nmask_per_seq <- as.integer(nmask_per_seq)
     masks_dirpath <- as.character(masks_dirpath)
     if (nmask_per_seq == 0L || length(seqnames) == 0L) {
@@ -342,11 +364,22 @@ MaskedBSgenome <- function(ref_bsgenome,
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### The 'show' method
+### Coercion
 ###
 
-.SHOW_BSGENOME_PREFIX <- "| "
-.SHOW_SEQSECTION_PREFIX <- "|   "
+setMethod("as.list", "BSgenome",
+    function(x)
+        lapply(setNames(seqnames(x), seqnames(x)),
+               function(seqname) x[[seqname]])
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### The "show" method
+###
+
+.SHOW_BSGENOME_PREFIX <- "# "
+.SHOW_SEQSECTION_PREFIX <- "#   "
 
 setMethod("show", "BSgenome",
     function(object)
@@ -354,48 +387,39 @@ setMethod("show", "BSgenome",
         mystrwrap <- function(line)
             writeLines(strwrap(line, width=getOption("width")+1,
                                exdent=0L, prefix=.SHOW_BSGENOME_PREFIX))
-        showSequenceIndex <- function(names, prefix)
-        {
-            index_width <- getOption("width") + 2L - nchar(prefix)
-            col_width <- max(nchar(names))
-            ncols <- index_width %/% (col_width + 2L)
-            col <- 1L
-            for (name in names) {
-                if (col == 1L) cat(prefix)
-                cat(format(name, width=col_width))
-                if (col == ncols) {
-                    cat("\n")
-                    col <- 1L
-                } else {
-                    cat("  ")
-                    col <- col + 1L
-                }
-            }
-            if (col != 1L) cat("\n")
-        }
-        if (!is.na(object@species)) {
-            cat(object@species, "genome\n")
-            cat(.SHOW_BSGENOME_PREFIX, "\n", sep="")
-        }
-        showGenomeDescription(object, margin=.SHOW_BSGENOME_PREFIX)
+        if (!is.na(object@common_name))
+            cat(object@common_name, "genome:\n")
+        GenomeInfoDb:::showGenomeDescription(object,
+                                             margin=.SHOW_BSGENOME_PREFIX)
         if (!is.null(SNPlocs_pkgname(object)))
             cat(.SHOW_BSGENOME_PREFIX, "with SNPs injected from package: ", SNPlocs_pkgname(object), "\n", sep="")
-        cat(.SHOW_BSGENOME_PREFIX, "\n", sep="")
-        if (length(mseqnames(object)) != 0L)
-            mystrwrap("single sequences (see '?seqnames'):")
-        else
-            mystrwrap("sequences (see '?seqnames'):")
-        if (length(seqnames(object)) != 0L)
-            showSequenceIndex(seqnames(object), .SHOW_SEQSECTION_PREFIX)
-        else
-            cat(.SHOW_SEQSECTION_PREFIX, "NONE\n", sep="")
-        cat(.SHOW_BSGENOME_PREFIX, "\n", sep="")
-        if (length(mseqnames(object)) != 0L) {
-            mystrwrap("multiple sequences (see '?mseqnames'):")
-            showSequenceIndex(mseqnames(object), .SHOW_SEQSECTION_PREFIX)
+        if (length(mseqnames(object)) == 0L) {
+            what <- "sequence"
+        } else {
             cat(.SHOW_BSGENOME_PREFIX, "\n", sep="")
+            what <- "single sequence"
+        }
+        mystrwrap(paste0(length(seqnames(object)), " ", what, "s:"))
+        if (length(seqnames(object)) != 0L) {
+            printAtomicVectorInAGrid(seqnames(object),
+                                     prefix=.SHOW_SEQSECTION_PREFIX)
+            mystrwrap(paste0("(use 'seqnames()' to see all the ",
+                             what, " names, ",
+                             "use the '$' or '[[' operator to access a given ",
+                             what, ")"))
+        } else {
+            cat(.SHOW_SEQSECTION_PREFIX, "NONE\n", sep="")
+        }
+        if (length(mseqnames(object)) != 0L) {
+            cat(.SHOW_BSGENOME_PREFIX, "\n", sep="")
+            mystrwrap("multiple sequences:")
+            printAtomicVectorInAGrid(mseqnames(object),
+                                     prefix=.SHOW_SEQSECTION_PREFIX)
+            mystrwrap(paste0("(use 'mseqnames()' to see all the ",
+                             "multiple sequence names, ",
+                             "use the '$' or '[[' operator to access a given ",
+                             "multiple sequence)"))
         }
-        mystrwrap("(use the '$' or '[[' operator to access a given sequence)")
     }
 )
 
@@ -410,8 +434,8 @@ setMethod("show", "BSgenome",
     if (is.na(idx)) {  # multiple sequence
         if (substr(seqname, 1L, 8L) == "upstream") {
             msg <- c(
-                "  Starting with BioC 2.14, upstream sequences ",
-                "are deprecated.\n",
+                "  Starting with BioC 3.0, the upstream sequences ",
+                "are defunct.\n",
                 "  However they can easily be extracted ",
                 "from the full genome\n  sequences with something like ",
                 "(for example for hg19):\n\n",
@@ -433,7 +457,7 @@ setMethod("show", "BSgenome",
                 "  makeTranscriptDbFromGFF() functions in the ",
                 "GenomicFeatures package."
             )
-            .Deprecated(msg=paste0(msg, collapse=""))
+            .Defunct(msg=paste0(msg, collapse=""))
         }
         ans <- x@multiple_sequences[[seqname]]
         return(ans)
@@ -446,7 +470,7 @@ setMethod("show", "BSgenome",
              "May be the data on disk is corrupted?")
     }
     ## Inject the SNPs, if any
-    snps <- SNPlocs(x, seqname)
+    snps <- snplocs(x, seqname)
     if (!is.null(snps))
         .inplaceReplaceLetterAt(ans, snps$loc, snps$alleles_as_ambig)
     ## Load and set the built-in masks, if any
@@ -556,14 +580,28 @@ setMethod("[[", "BSgenome",
     }
 )
 
-setReplaceMethod("[[", "BSgenome",
-    function(x, i, j,..., value)
-    {
-        stop("attempt to modify the value of a \"BSgenome\" object")
-    }
-)
-
 setMethod("$", "BSgenome",
     function(x, name) x[[name]]
 )
 
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Old stuff (deprecated & defunct)
+###
+
+setMethod("SNPcount", "BSgenome",
+    function(x)
+    {
+        .Deprecated("snpcount")
+        snpcount(x)
+    }
+)
+
+setMethod("SNPlocs", "BSgenome",
+    function(x, seqname)
+    {
+        .Deprecated("snplocs")
+        snplocs(x, seqname)
+    }
+)
+
