@@ -44,6 +44,52 @@ setClass("SNPlocs",
 )
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Accessors
+###
+
+setMethod("provider", "SNPlocs", function(x) x@provider)
+
+setMethod("providerVersion", "SNPlocs", function(x) x@provider_version)
+
+setMethod("releaseDate", "SNPlocs", function(x) x@release_date)
+
+setMethod("releaseName", "SNPlocs", function(x) x@release_name)
+
+setGeneric("referenceGenome", function(x) standardGeneric("referenceGenome"))
+
+setMethod("referenceGenome", "SNPlocs", function(x) x@reference_genome)
+
+setGeneric("compatibleGenomes",
+    function(x) standardGeneric("compatibleGenomes")
+)
+setMethod("compatibleGenomes", "SNPlocs", function(x) x@compatible_genomes)
+
+setMethod("organism", "SNPlocs",
+    function(object) organism(referenceGenome(object))
+)
+
+setMethod("commonName", "SNPlocs",
+    function(object) commonName(referenceGenome(object))
+)
+
+setMethod("species", "SNPlocs",
+    function(object)
+    {
+         msg <- wmsg("The \"species\" method for SNPlocs objects ",
+                     "is deprecated and should not be used anymore. ",
+                     "Please use commonName() instead of species() on ",
+                     "SNPlocs objects.")
+        .Deprecated(msg=msg)
+        commonName(object)
+    }
+)
+
+setMethod("seqinfo", "SNPlocs", function(x) seqinfo(referenceGenome(x)))
+
+setMethod("seqnames", "SNPlocs", function(x) seqnames(referenceGenome(x)))
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Constructor
 ###
@@ -76,35 +122,6 @@ newSNPlocs <- function(provider, provider_version,
 }
 
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Accessors
-###
-
-setMethod("provider", "SNPlocs", function(x) x@provider)
-
-setMethod("providerVersion", "SNPlocs", function(x) x@provider_version)
-
-setMethod("releaseDate", "SNPlocs", function(x) x@release_date)
-
-setMethod("releaseName", "SNPlocs", function(x) x@release_name)
-
-setGeneric("referenceGenome", function(x) standardGeneric("referenceGenome"))
-setMethod("referenceGenome", "SNPlocs", function(x) x@reference_genome)
-
-setGeneric("compatibleGenomes",
-    function(x) standardGeneric("compatibleGenomes")
-)
-setMethod("compatibleGenomes", "SNPlocs", function(x) x@compatible_genomes)
-
-setMethod("organism", "SNPlocs", function(x) organism(referenceGenome(x)))
-
-setMethod("species", "SNPlocs", function(x) species(referenceGenome(x)))
-
-setMethod("seqinfo", "SNPlocs", function(x) seqinfo(referenceGenome(x)))
-
-setMethod("seqnames", "SNPlocs", function(x) seqnames(referenceGenome(x)))
-
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### The 'show' method
 ###
@@ -112,14 +129,17 @@ setMethod("seqnames", "SNPlocs", function(x) seqnames(referenceGenome(x)))
 setMethod("show", "SNPlocs",
     function(object)
     {
-        cat(class(object), " object for ", organism(object), " (",
-            provider(object), " ", releaseName(object), ")\n", sep="")
+        cat("# ", class(object), " object for ", organism(object),
+            " (", releaseName(object), ")\n", sep="")
+        cat("# reference genome: ",
+            providerVersion(referenceGenome(object)), "\n", sep="")
+        cat("# nb of SNPs: ", sum(snpcount(object)), "\n", sep="")
     }
 )
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### snpcount() and snplocs()
+### snpcount()
 ###
 
 ### Creates objects "on-the-fly" (not serialized).
@@ -163,6 +183,7 @@ setMethod("show", "SNPlocs",
 }
 
 setGeneric("snpcount", function(x) standardGeneric("snpcount"))
+
 setMethod("snpcount", "SNPlocs",
     function(x)
     {
@@ -176,6 +197,11 @@ setMethod("snpcount", "SNPlocs",
     }
 )
 
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### OLD API: snplocs(), snpid2loc(), snpid2alleles(), and snpid2grange()
+###
+
 ### Load raw snplocs.
 .load_raw_snplocs <- function(x, seqname, caching)
 {
@@ -294,16 +320,11 @@ setMethod("snplocs", "SNPlocs",
     }
 )
 
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### snpid2loc(), snpid2alleles(), and snpid2grange()
-###
-
 .normargSnpid <- function(snpid)
 {
     if (!is.vector(snpid))
         stop("'snpid' must be an integer or character vector")
-    if (IRanges:::anyMissing(snpid))
+    if (S4Vectors:::anyMissing(snpid))
         stop("'snpid' cannot contain NAs")
     if (is.numeric(snpid)) {
         if (!is.integer(snpid))
@@ -321,7 +342,7 @@ setMethod("snplocs", "SNPlocs",
         snpid <- substr(snpid, 3, nchar(snpid))
     }
     snpid <- suppressWarnings(as.integer(snpid))
-    if (IRanges:::anyMissing(snpid))
+    if (S4Vectors:::anyMissing(snpid))
         stop("cannot extract the digital part of some ids in 'snpid'")
     snpid
 }
@@ -341,7 +362,13 @@ setMethod("snplocs", "SNPlocs",
         if (length(bad_snpid_idx) != 0L) {
             bad_snpid <- snpid[bad_snpid_idx]
             bad_snpid_in1string <- paste(bad_snpid, collapse=", ")
-            stop("SNP id(s) not found: ", bad_snpid_in1string)
+            stop(wmsg("SNP id(s) not found: ", bad_snpid_in1string,
+                      "\n\nPlease note that rsid2loc(), snpid2loc(), ",
+                      "rsid2alleles(), snpid2alleles(), rsidsToGRanges(), ",
+                      "and snpid2grange() are superseded by snpsById(). ",
+                      "The latter can be called with 'ifnotfound=\"drop\"' ",
+                      "to drop unknown SNP ids. See '?snpsById' for more ",
+                      "information.\n\n "))
         }
     }
     seqidx <- findInterval(idx - 1L, cumsum(snpcount(x))) + 1L
@@ -432,3 +459,169 @@ setMethod("snpid2grange", "SNPlocs",
     }
 )
 
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### NEW API: snpsBySeqname(), snpsByOverlaps(), snpsById() (similar to API
+### for querying an XtraSNPlocs object)
+###
+
+setGeneric("snpsBySeqname", signature="x",
+    function(x, seqnames, ...) standardGeneric("snpsBySeqname")
+)
+
+### Same args and signature as GenomicFeatures::transcriptsByOverlaps()
+### EXCEPT for 'minoverlap' default value that we set to zero so we also
+### get SNPs that are insertions (relevant for XtraSNPlocs objects).
+setGeneric("snpsByOverlaps", signature="x",
+    function(x, ranges, maxgap=0L, minoverlap=0L,
+             type=c("any", "start", "end", "within", "equal"), ...)
+        standardGeneric("snpsByOverlaps")
+)
+
+setGeneric("snpsById", signature="x",
+    function(x, ids, ...) standardGeneric("snpsById")
+)
+
+.get_GRanges_by_seqname_from_SNPlocs <- function(x, seqnames, drop.rs.prefix)
+{
+    gr <- snplocs(x, seqnames, as.GRanges=TRUE)
+    if (!drop.rs.prefix && length(gr) != 0L)
+        mcols(gr)$RefSNP_id <- paste0("rs", mcols(gr)$RefSNP_id)
+    gr
+}
+
+.SNPlocs_snpsBySeqname <- function(x, seqnames, drop.rs.prefix=FALSE)
+{
+    if (!is.character(seqnames)
+     || any(is.na(seqnames))
+     || any(duplicated(seqnames)))
+        stop(wmsg("'seqnames' must be a character vector ",
+                  "with no NAs and no duplicates"))
+    if (!all(seqnames %in% seqlevels(x)))
+        stop(wmsg("'seqnames' must be a subset of: ",
+                  paste(seqlevels(x), collapse=", ")))
+    if (!isTRUEorFALSE(drop.rs.prefix))
+        stop(wmsg("'drop.rs.prefix' must be TRUE or FALSE"))
+    .get_GRanges_by_seqname_from_SNPlocs(x, seqnames, drop.rs.prefix)
+}
+
+setMethod("snpsBySeqname", "SNPlocs", .SNPlocs_snpsBySeqname)
+
+### TODO: Avoid code duplication between normarg_ranges() and
+### GenomicAlignments:::.normarg_param().
+normarg_ranges <- function(ranges)
+{
+    if (isSingleString(ranges)) {
+        tmp1 <- strsplit(ranges, ":", fixed=TRUE)[[1L]]
+        if (length(tmp1) != 2L)
+            stop(wmsg("when a character string, 'ranges' must be ",
+                      "of the form \"ch14:5201-5300\""))
+        tmp2 <- as.integer(strsplit(tmp1[2L], "-", fixed=TRUE)[[1L]])
+        if (length(tmp2) != 2L || any(is.na(tmp2)))
+            stop(wmsg("when a character string, 'ranges' must be ",
+                      "of the form \"ch14:5201-5300\""))
+        ranges <- GRanges(tmp1[1L], IRanges(tmp2[1L], tmp2[2L]))
+        return(ranges)
+    }
+    if (!is(ranges, "GenomicRanges"))
+        stop(wmsg("'ranges' ranges must be a GenomicRanges object ",
+                  "or a character string of the form \"ch14:5201-5300\""))
+    ranges
+}
+
+.SNPlocs_snpsByOverlaps <- function(x, ranges, maxgap=0L, minoverlap=0L,
+             type=c("any", "start", "end", "within", "equal"),
+             drop.rs.prefix=FALSE, ...)
+{
+    ranges <- normarg_ranges(ranges)
+    ## The only purpose of the line below is to check that 'x' and 'ranges'
+    ## are based on the same reference genome (merge() will raise an error
+    ## if they are not).
+    merge(seqinfo(x), seqinfo(ranges))
+    seqlevels(ranges, force=TRUE) <- intersect(seqlevels(x),
+                                               seqlevelsInUse(ranges))
+    snps_by_seqname <- .SNPlocs_snpsBySeqname(x, seqlevels(ranges),
+                                              drop.rs.prefix=drop.rs.prefix)
+    subsetByOverlaps(snps_by_seqname, ranges,
+                     maxgap=maxgap, minoverlap=minoverlap,
+                     type=type, ...)
+}
+
+setMethod("snpsByOverlaps", "SNPlocs", .SNPlocs_snpsByOverlaps)
+
+### Return an integer vector with no NAs parallel to 'ids'.
+ids2rowids <- function(ids)
+{
+    if (!(is.character(ids) || is.numeric(ids)))
+        stop(wmsg("'ids' must be a character or integer vector with no NAs"))
+    if (S4Vectors:::anyMissing(ids))
+        stop(wmsg("'ids' cannot contain NAs"))
+    if (is.character(ids)) {
+        prefixes <- unique(substr(ids, 1L, 2L))
+        if ("rs" %in% prefixes) {
+            if (!setequal(prefixes, "rs"))
+                stop(wmsg("'ids' cannot mix SNP ids that are prefixed ",
+                          "with \"rs\" with SNP ids that are not"))
+            ## Drop the "rs" prefix.
+            ids <- substr(ids, 3L, nchar(ids))
+        }
+        ids <- suppressWarnings(as.numeric(ids))
+        if (S4Vectors:::anyMissing(ids))
+            stop(wmsg("cannot extract the digital part of ",
+                      "some SNP ids in 'ids'"))
+    }
+    if (length(ids) != 0L && min(ids) < 0)
+        stop(wmsg("'ids' contains unrealistic SNP ids"))
+    if (!is.integer(ids)) {
+        ids <- suppressWarnings(as.integer(ids))
+        if (S4Vectors:::anyMissing(ids))
+            stop(wmsg("'ids' contains SNP ids that are too big"))
+    }
+    ids
+}
+
+### Return a list of 2 vectors parallel to each other. The 1st and 2nd vectors
+### are respectively the row indices (integer vector) and the 'user_ids' vector
+### (which can be character, numeric, or integer), with the not found entries
+### removed from both of them.
+### Note that, if 'ifnotfound="error"' then the 2 returned vectors are parallel
+### to input vectors 'user_rowids' and 'user_ids'.
+rowids2rowidx <- function(user_rowids, user_ids, x_rowids, ifnotfound)
+{
+    if (is.null(x_rowids))
+        stop(wmsg("BSgenome internal error: data contains no SNP ids"))
+    rowidx <- match(user_rowids, x_rowids)
+    notfound_idx <- which(is.na(rowidx))
+    if (length(notfound_idx) != 0L) {
+        if (length(notfound_idx) <= 10L) {
+            ids_to_show <- user_ids[notfound_idx]
+        } else {
+            ids_to_show <- c(user_ids[notfound_idx[1:9]], "...")
+        }
+        ids_to_show <- paste0(ids_to_show, collapse=", ")
+        if (ifnotfound == "error")
+            stop(wmsg("SNP ids not found: ",
+                      ids_to_show,
+                      "\n\nUse 'ifnotfound=\"drop\"' to drop them."))
+        if (ifnotfound == "warning")
+            warning(wmsg("SNP ids not found: ", ids_to_show,
+                         "\n\nThey were dropped."))
+        rowidx <- rowidx[-notfound_idx]
+        user_ids <- user_ids[-notfound_idx]
+    }
+    list(rowidx, user_ids)
+}
+
+.SNPlocs_snpsById <- function(x, ids, ifnotfound=c("error", "warning", "drop"))
+{
+    user_rowids <- ids2rowids(ids)
+    ifnotfound <- match.arg(ifnotfound)
+    x_rowids <- .get_SNPlocs_data(x, "all_rsids")
+    rowidx <- rowids2rowidx(user_rowids, ids, x_rowids, ifnotfound)
+    ans <- snpid2grange(x, rowidx[[1L]])
+    mcols(ans)[ , "RefSNP_id"] <- rowidx[[2L]]
+    ans
+}
+
+setMethod("snpsById", "SNPlocs", .SNPlocs_snpsById)
+
