@@ -23,7 +23,7 @@
     if (length(strand) < length) {
         if (length(strand) == 0L)
             stop("cannot recycle zero-length 'strand'")
-        strand <- IRanges:::recycleVector(strand, length)
+        strand <- S4Vectors:::recycleVector(strand, length)
     }
     strand
 }
@@ -58,10 +58,10 @@
     l3 <- length(width)
     max0123 <- max(l0, l1, l2, l3)
     ## Recycling will fail for vectors of length 0.
-    names <- IRanges:::recycleVector(names, max0123)
-    start <- IRanges:::recycleVector(start, max0123)
-    end <- IRanges:::recycleVector(end, max0123)
-    width <- IRanges:::recycleVector(width, max0123)
+    names <- S4Vectors:::recycleVector(names, max0123)
+    start <- S4Vectors:::recycleVector(start, max0123)
+    end <- S4Vectors:::recycleVector(end, max0123)
+    width <- S4Vectors:::recycleVector(width, max0123)
     strand <- .starfreeStrand(.normargStrand(strand, max0123))
     list(names=names, start=start, end=end, width=width, strand=strand)
 }
@@ -86,27 +86,6 @@
 ### .toGRanges() is called.
 .toGRanges <- function(x, strand)
 {
-    if (is(x, "RangedData")) {
-        msg <- c("Starting with BioC 2.14, passing a RangedData object ",
-                 "to getSeq() is deprecated. Please use a GRanges object ",
-                 "instead. Note that you can obtain a GRanges object by ",
-                 "simply coercing the RangedData object with ",
-                 "'as(..., \"RangedData\")'. However, we strongly recommend ",
-                 "that you start migrating your code to operate on GRanges ",
-                 "objects instead of RangedData objects. Please ask on the ",
-                 "bioc-devel mailing list if you have questions or concerns ",
-                 "about this (http://bioconductor.org/help/mailing-list/).")
-        .Deprecated(msg=msg)
-        if ("strand" %in% colnames(x)) {
-            if (!identical(strand, "+"))
-                stop("'strand' cannot be specified when 'names' ",
-                     "is a RangedData object with a strand column")
-        } else {
-            strand <- Rle(strand(.normargStrand(strand, nrow(x))))
-            values(x) <- DataFrame(strand=strand)
-        }
-        return(as(x, "GRanges"))
-    }
     if (is(x, "RangesList") || is(x, "Ranges")) {
         if (is.null(names(x)))
             stop("when 'names' is a RangesList or Ranges object, ",
@@ -120,14 +99,10 @@
     stop("invalid 'names'")
 }
 
-.dropUnusedGRangesSeqnamesLevels <- function(x)
+.drop_unused_seqlevels <- function(x)
 {
-    ## There should be a better way to drop unused levels!
-    seqnames <- as.character(seqnames(x))
-    GRanges(seqnames=seqnames,
-            ranges=ranges(x),
-            strand=strand(x),
-            seqlengths=seqlengths(x)[unique(seqnames)])
+    seqlevels(x) <- seqlevelsInUse(x)
+    x
 }
 
 
@@ -143,6 +118,13 @@
                                                 start, end, width, strand)
 {
     if (is.character(names)) {
+        #idx <- which(is.na(start) & is.na(width))
+        #if (length(idx) != 0L)
+        #    start[idx] <- 1L
+        #idx <- which(is.na(end) & is.na(width))
+        #if (length(idx) != 0L)
+        #    end[idx] <- unname(seqlengths(x)[names])
+        #ranges <- IRanges(start=start, end=end, width=width)
         refwidths <- seqlengths(x)[names]
         ranges <- solveUserSEW(refwidths, start=start, end=end, width=width)
         names <- GRanges(seqnames=names, ranges=ranges, strand=strand)
@@ -174,7 +156,7 @@
             if (is.na(idx))
                 stop("invalid sequence name: ", seqlevel)
             seqname <- names(x@user_seqnames)[[idx]]
-            if (is.null(SNPlocs(x, seqname))) {
+            if (is.null(snplocs(x, seqname))) {
                 ## Try to grab the subject sequence from the cache.
                 subject <- try(get(seqname, envir=x@.seqs_cache,
                                    inherits=FALSE), silent=TRUE)
@@ -342,8 +324,7 @@ setMethod("getSeq", "BSgenome",
             strand(names) <- .starfreeStrand(strand(names))
             seqnames <- as.character(seqnames(names))
             sseq_idx <- seqnames %in% seqnames(x)
-            sseq_args <- list(names=.dropUnusedGRangesSeqnamesLevels(
-                                    names[sseq_idx]))
+            sseq_args <- list(names=.drop_unused_seqlevels(names[sseq_idx]))
             mseq_idx <- !sseq_idx
             mseq_args <- list(names=names[mseq_idx])
         }
