@@ -15,6 +15,7 @@ BSgenome_datapkgs <- c(
   "BSgenome.Btaurus.UCSC.bosTau3",
   "BSgenome.Btaurus.UCSC.bosTau4",
   "BSgenome.Btaurus.UCSC.bosTau6",
+  "BSgenome.Btaurus.UCSC.bosTau8",
   "BSgenome.Celegans.UCSC.ce2",
   "BSgenome.Celegans.UCSC.ce6",
   "BSgenome.Celegans.UCSC.ce10",
@@ -22,6 +23,7 @@ BSgenome_datapkgs <- c(
   "BSgenome.Cfamiliaris.UCSC.canFam3",
   "BSgenome.Dmelanogaster.UCSC.dm2",
   "BSgenome.Dmelanogaster.UCSC.dm3",
+  "BSgenome.Dmelanogaster.UCSC.dm6",
   "BSgenome.Drerio.UCSC.danRer5",
   "BSgenome.Drerio.UCSC.danRer6",
   "BSgenome.Drerio.UCSC.danRer7",
@@ -33,6 +35,8 @@ BSgenome_datapkgs <- c(
   "BSgenome.Hsapiens.UCSC.hg18",
   "BSgenome.Hsapiens.UCSC.hg19",
   "BSgenome.Hsapiens.NCBI.GRCh38",
+  "BSgenome.Hsapiens.UCSC.hg38",
+  "BSgenome.Mfuro.UCSC.musFur1",
   "BSgenome.Mmulatta.UCSC.rheMac2",
   "BSgenome.Mmulatta.UCSC.rheMac3",
   "BSgenome.Mmusculus.UCSC.mm8",
@@ -42,6 +46,7 @@ BSgenome_datapkgs <- c(
   "BSgenome.Ptroglodytes.UCSC.panTro3",
   "BSgenome.Rnorvegicus.UCSC.rn4",
   "BSgenome.Rnorvegicus.UCSC.rn5",
+  "BSgenome.Rnorvegicus.UCSC.rn6",
   "BSgenome.Scerevisiae.UCSC.sacCer1",
   "BSgenome.Scerevisiae.UCSC.sacCer2",
   "BSgenome.Scerevisiae.UCSC.sacCer3",
@@ -54,7 +59,7 @@ for (pkg in BSgenome_datapkgs) {
   cat("============================================================\n")
   cat("START FORGING ", pkg, "\n", sep="")
   cat("\n")
-  forgeBSgenomeDataPkg(pkg, mode="fa.rz")
+  forgeBSgenomeDataPkg(pkg)
   cat("\n")
   cat("END FORGING ", pkg, "\n", sep="")
 }
@@ -77,6 +82,7 @@ MaskedBSgenome_datapkgs <- c(
   "BSgenome.Hsapiens.UCSC.hg17.masked",
   "BSgenome.Hsapiens.UCSC.hg18.masked",
   "BSgenome.Hsapiens.UCSC.hg19.masked",
+  "BSgenome.Hsapiens.UCSC.hg38.masked",
   "BSgenome.Mmulatta.UCSC.rheMac2.masked",
   "BSgenome.Mmulatta.UCSC.rheMac3.masked",
   "BSgenome.Mmusculus.UCSC.mm8.masked",
